@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2009 Andrew Apted
+//  Copyright (C) 2001-2016 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -28,8 +28,7 @@
 
 #include "m_game.h"
 #include "e_things.h"
-#include "editloop.h"
-#include "levels.h"
+#include "e_main.h"
 #include "m_bitvec.h"
 #include "w_rawdef.h"
 
@@ -47,10 +46,10 @@ int calc_new_angle(int angle, int diff)
 }
 
 
-/*
- *  spin_thing - change the angle of things
- */
-void TH_SpinThings(void)
+//
+// spin_thing - change the angle of things
+//
+void CMD_TH_SpinThings(void)
 {
 	int degrees = atoi(EXEC_Param[0]);
 
@@ -75,6 +74,8 @@ void TH_SpinThings(void)
 		BA_ChangeTH(*it, Thing::F_ANGLE, calc_new_angle(T->angle, degrees));
 	}
 
+	BA_MessageForSel("spun", &list);
+
 	BA_End();
 
 	main_win->thing_box->UpdateField(Thing::F_ANGLE);
@@ -136,11 +137,12 @@ static void MoveOverlapThing(int th, int mid_x, int mid_y, int n, int total)
 }
 
 
-/*  all things lying at same location (or very near) to the selected
- *  things are moved so they are more distinct -- about 8 units away
- *  from that location.
- */
-void TH_Disconnect(void)
+//
+//  all things lying at same location (or very near) to the selected
+//  things are moved so they are more distinct -- about 8 units away
+//  from that location.
+//
+void CMD_TH_Disconnect(void)
 {
 	if (edit.Selected->empty())
 	{
@@ -155,6 +157,8 @@ void TH_Disconnect(void)
 
 	BA_Begin();
 
+	BA_MessageForSel("disconnected", edit.Selected);
+
 	while (! edit.Selected->empty())
 	{
 		selection_c overlaps(OBJ_THINGS);
@@ -172,7 +176,7 @@ void TH_Disconnect(void)
 			continue;
 
  		int mid_x, mid_y;
- 
+
  		Objs_CalcMiddle(&overlaps, &mid_x, &mid_y);
 
 
@@ -188,8 +192,10 @@ void TH_Disconnect(void)
 }
 
 
-/* place all selected things at same location */
-void TH_Merge(void)
+//
+// place all selected things at same location
+//
+void CMD_TH_Merge(void)
 {
 	if (edit.Selected->count_obj() == 1 && edit.highlight.valid())
 	{
@@ -208,6 +214,8 @@ void TH_Merge(void)
 
 	BA_Begin();
 
+	BA_MessageForSel("merged", edit.Selected);
+
 	selection_iterator_c it;
 
 	for (edit.Selected->begin(&it) ; !it.at_end() ; ++it)
