@@ -54,7 +54,7 @@ public:
 public:
 	Objid() : type(OBJ_THINGS), num(NIL_OBJ) { }
 	Objid(obj_type_e t, int n) : type(t), num(n) { }
-	Objid(const Objid& other) : type(other.type), num(other.num) { } 
+	Objid(const Objid& other) : type(other.type), num(other.num) { }
 
 	void clear() { num = NIL_OBJ; }
 
@@ -72,6 +72,70 @@ public:
 	bool is_nil() const { return num <  0; }
 };
 
+
+//------------------------------------------------------------------------
+//   ITEMS IN THE 3D VIEW
+//------------------------------------------------------------------------
+
+
+typedef enum
+{
+	OB3D_Thing = 0,
+
+	OB3D_Floor = -1,
+	OB3D_Ceil  = -2,
+
+	OB3D_Lower = 1,  // used for middle of 1S lines too
+	OB3D_Rail  = 2,
+	OB3D_Upper = 3,
+
+} obj3d_type_e;
+
+
+class Obj3d_t
+{
+public:
+	obj3d_type_e type;
+
+	// the linedef, sector or thing number, or NIL_OBJ of none
+	int num;
+
+	// SIDE_XXX of the line  [ unused for sectors or things ]
+	int side;
+
+public:
+	Obj3d_t() : type(OB3D_Thing), num(NIL_OBJ), side()
+	{ }
+
+	Obj3d_t(const Obj3d_t& other) :
+		type(other.type),
+		num (other.num),
+		side(other.side)
+	{ }
+
+	void clear()
+	{
+		type  = OB3D_Thing;
+		num   = NIL_OBJ;
+		side  =  0;
+	}
+
+	bool valid()  const { return num >= 0; }
+	bool is_nil() const { return num <  0; }
+
+	bool isLine()   const { return valid() && (type >= OB3D_Lower); }
+	bool isSector() const { return valid() && (type <= OB3D_Floor); }
+	bool isThing()  const { return valid() && (type == OB3D_Thing); }
+
+	bool operator== (const Obj3d_t& other) const
+	{
+		return	(type == other.type) &&
+				(num  == other.num)  &&
+				(side == other.side);
+	}
+};
+
+
 #endif  /* __EUREKA_OBJ_ID_H__ */
 
 //--- editor settings ---
