@@ -21,20 +21,24 @@
 #include "main.h"
 #include "ui_window.h"
 
-#include "levels.h"
+#include "e_hover.h"	// OppositeSector
+#include "e_linedef.h"
+#include "e_main.h"
+#include "m_game.h"
+#include "r_render.h"
 #include "w_rawdef.h"
-#include "x_hover.h"
-#include "x_loop.h"
+#include "w_texture.h"
 
 
 // config item
-bool swap_sidedefs;
-bool show_full_one_sided;
+bool swap_sidedefs = false;
+bool show_full_one_sided = false;
+bool sidedef_add_del_buttons = false;
 
 //
 // Constructor
 //
-UI_SideBox::UI_SideBox(int X, int Y, int W, int H, int _side) : 
+UI_SideBox::UI_SideBox(int X, int Y, int W, int H, int _side) :
     Fl_Group(X, Y, W, H),
     obj(SETOBJ_NO_LINE), is_front(_side == 0),
 	on_2S_line(false)
@@ -48,7 +52,7 @@ UI_SideBox::UI_SideBox(int X, int Y, int W, int H, int _side) :
 	else
 		labelcolor(fl_rgb_color(224,64,0));
 
-	
+
 	add_button = new Fl_Button(X + W - 120, Y, 50, 20, "ADD");
 	add_button->labelcolor(labelcolor());
 	add_button->callback(add_callback, this);
@@ -89,7 +93,7 @@ UI_SideBox::UI_SideBox(int X, int Y, int W, int H, int _side) :
 	int LX = X+16;
 	int UX = X+W-64-16;
 	    MX = MX-32;
-	
+
 	if (swap_sidedefs)
 	{
 		std::swap(UX, LX);
@@ -107,9 +111,9 @@ UI_SideBox::UI_SideBox(int X, int Y, int W, int H, int _side) :
 	Y += 65;
 
 
-	l_tex = new Fl_Input(LX-8, Y, 80, 20);
-	u_tex = new Fl_Input(UX-8, Y, 80, 20);
-	r_tex = new Fl_Input(MX-8, Y, 80, 20);
+	l_tex = new UI_DynInput(LX-8, Y, 80, 20);
+	u_tex = new UI_DynInput(UX-8, Y, 80, 20);
+	r_tex = new UI_DynInput(MX-8, Y, 80, 20);
 
 	l_tex->textsize(12);
 	u_tex->textsize(12);
@@ -119,6 +123,10 @@ UI_SideBox::UI_SideBox(int X, int Y, int W, int H, int _side) :
 	u_tex->callback(tex_callback, this);
 	r_tex->callback(tex_callback, this);
 
+	l_tex->callback2(dyntex_callback, this);
+	u_tex->callback2(dyntex_callback, this);
+	r_tex->callback2(dyntex_callback, this);
+
 	l_tex->when(FL_WHEN_RELEASE | FL_WHEN_ENTER_KEY);
 	u_tex->when(FL_WHEN_RELEASE | FL_WHEN_ENTER_KEY);
 	r_tex->when(FL_WHEN_RELEASE | FL_WHEN_ENTER_KEY);
@@ -137,8 +145,7 @@ UI_SideBox::UI_SideBox(int X, int Y, int W, int H, int _side) :
 // Destructor
 //
 UI_SideBox::~UI_SideBox()
-{
-}
+{ }
 
 
 void UI_SideBox::tex_callback(Fl_Widget *w, void *data)
@@ -154,10 +161,11 @@ void UI_SideBox::tex_callback(Fl_Widget *w, void *data)
 		UI_Pic * pic = (UI_Pic *)w;
 
 		pic->Selected(! pic->Selected());
-		pic->redraw();
+
+		Render3D_ClearSelection();
 
 		if (pic->Selected())
-			main_win->ShowBrowser('T');
+			main_win->BrowserMode('T');
 		return;
 	}
 
@@ -169,16 +177,16 @@ void UI_SideBox::tex_callback(Fl_Widget *w, void *data)
 		if (w == box->r_pic)
 			new_tex = BA_InternaliseString("-");
 		else
-			new_tex = BA_InternaliseString(default_lower_tex);
+			new_tex = BA_InternaliseString(default_wall_tex);
 	}
 	else
 	{
 		if (w == box->l_tex)
-			new_tex = TexFromWidget(box->l_tex);
+			new_tex = BA_InternaliseString(NormalizeTex(box->l_tex->value()));
 		else if (w == box->u_tex)
-			new_tex = TexFromWidget(box->u_tex);
+			new_tex = BA_InternaliseString(NormalizeTex(box->u_tex->value()));
 		else
-			new_tex = TexFromWidget(box->r_tex);
+			new_tex = BA_InternaliseString(NormalizeTex(box->r_tex->value()));
 	}
 
 	// iterate over selected linedefs
@@ -208,19 +216,18 @@ void UI_SideBox::tex_callback(Fl_Widget *w, void *data)
 				{
 					BA_ChangeSD(sd, SideDef::F_LOWER_TEX, new_tex);
 				}
-
-				if (upper)
+				else if (upper)
 				{
 					BA_ChangeSD(sd, SideDef::F_UPPER_TEX, new_tex);
 				}
-
-				if (rail)
+				else if (rail)
 				{
 					BA_ChangeSD(sd, SideDef::F_MID_TEX,   new_tex);
 				}
 			}
 		}
 
+		BA_MessageForSel("edited texture on", &list);
 		BA_End();
 
 		box->UpdateField();
@@ -228,6 +235,30 @@ void UI_SideBox::tex_callback(Fl_Widget *w, void *data)
 }
 
 
+void UI_SideBox::dyntex_callback(Fl_Widget *w, void *data)
+{
+	// change picture to match the input, BUT does not change the map
+
+	UI_SideBox *box = (UI_SideBox *)data;
+
+	if (box->obj < 0)
+		return;
+
+	if (w == box->l_tex)
+	{
+		box->l_pic->GetTex(box->l_tex->value());
+	}
+	else if (w == box->u_tex)
+	{
+		box->u_pic->GetTex(box->u_tex->value());
+	}
+	else if (w == box->r_tex)
+	{
+		box->r_pic->GetTex(box->r_tex->value());
+	}
+}
+
+
 void UI_SideBox::add_callback(Fl_Widget *w, void *data)
 {
 	UI_SideBox *box = (UI_SideBox *)data;
@@ -302,6 +333,7 @@ void UI_SideBox::add_callback(Fl_Widget *w, void *data)
 			LD_AddSecondSideDef(*it, sd, other);
 	}
 
+	BA_MessageForSel("added sidedef to", edit.Selected);
 	BA_End();
 
 	main_win->line_box->UpdateField();
@@ -338,6 +370,7 @@ void UI_SideBox::delete_callback(Fl_Widget *w, void *data)
 		LD_RemoveSideDef(*it, box->is_front ? SIDE_RIGHT : SIDE_LEFT);
 	}
 
+	BA_MessageForSel("deleted sidedef from", edit.Selected);
 	BA_End();
 
 	main_win->line_box->UpdateField();
@@ -375,6 +408,11 @@ void UI_SideBox::offset_callback(Fl_Widget *w, void *data)
 			}
 		}
 
+		if (w == box->x_ofs)
+			BA_MessageForSel("edited X offset on", &list);
+		else
+			BA_MessageForSel("edited Y offset on", &list);
+
 		BA_End();
 	}
 }
@@ -406,6 +444,7 @@ void UI_SideBox::sector_callback(Fl_Widget *w, void *data)
 				BA_ChangeSD(sd, SideDef::F_SECTOR, new_sec);
 		}
 
+		BA_MessageForSel("edited sector-ref on", &list);
 		BA_End();
 	}
 }
@@ -464,11 +503,15 @@ void UI_SideBox::UpdateField()
 		u_pic->GetTex(upper);
 		r_pic->GetTex(rail);
 
-		if ((what_is_solid & (SOLID_LOWER | SOLID_MID)) && (lower[0] == '-'))
+		if ((what_is_solid & (SOLID_LOWER | SOLID_MID)) && is_null_tex(lower))
 			l_pic->MarkMissing();
 
-		if ((what_is_solid & SOLID_UPPER) && (upper[0] == '-'))
+		if ((what_is_solid & SOLID_UPPER) && is_null_tex(upper))
 			u_pic->MarkMissing();
+
+		l_pic->AllowHighlight(true);
+		u_pic->AllowHighlight(true);
+		r_pic->AllowHighlight(true);
 	}
 	else
 	{
@@ -483,6 +526,10 @@ void UI_SideBox::UpdateField()
 		l_pic->Clear();
 		u_pic->Clear();
 		r_pic->Clear();
+
+		l_pic->AllowHighlight(false);
+		u_pic->AllowHighlight(false);
+		r_pic->AllowHighlight(false);
 	}
 }
 
@@ -506,7 +553,7 @@ void UI_SideBox::UpdateLabel()
 
 void UI_SideBox::UpdateAddDel()
 {
-	if (obj == SETOBJ_NO_LINE)
+	if (obj == SETOBJ_NO_LINE || ! sidedef_add_del_buttons)
 	{
 		add_button->hide();
 		del_button->hide();
@@ -571,11 +618,22 @@ void UI_SideBox::UpdateHiding()
 
 int UI_SideBox::GetSelectedPics() const
 {
+	if (obj < 0) return 0;
+
 	return	(l_pic->Selected() ? 1 : 0) |
 			(u_pic->Selected() ? 2 : 0) |
 			(r_pic->Selected() ? 4 : 0);
 }
 
+int UI_SideBox::GetHighlightedPics() const
+{
+	if (obj < 0) return 0;
+
+	return	(l_pic->Highlighted() ? 1 : 0) |
+			(u_pic->Highlighted() ? 2 : 0) |
+			(r_pic->Highlighted() ? 4 : 0);
+}
+
 
 void UI_SideBox::UnselectPics()
 {
@@ -585,20 +643,5 @@ void UI_SideBox::UnselectPics()
 }
 
 
-int UI_SideBox::TexFromWidget(Fl_Input *w)
-{
-	char name[WAD_TEX_NAME+1];
-
-	memset(name, 0, sizeof(name));
-
-	strncpy(name, w->value(), WAD_TEX_NAME);
-
-	for (int i = 0 ; i < WAD_TEX_NAME ; i++)
-		name[i] = toupper(name[i]);
-
-	return BA_InternaliseString(name);
-}
-
-
 //--- editor settings ---
 // vi:ts=4:sw=4:noexpandtab
